<script src="https://polyfill.io/v3/polyfill.min.js?features=es6"></script>
<link rel="stylesheet" href="{{ "/assets/css/default.css" | relative_url }}">
{% include highlight/highlight_header.html %}
{% include mathjax/mathjax.html %}
{% include head-custom2.html %}