---
layout: page
---

{%- include toppage_header.html -%}

{%- if content -%}
    {{- content -}}
{%- else -%}
    <p>
    {%- if site.github.project_tagline -%}
        {{- site.github.project_tagline -}}
    {%- else -%}
        This documentation is automatically generated by <a href="https://github.com/competitive-verifier/competitive-verifier">competitive-verifier/competitive-verifier</a>.
    {%- endif -%}
    </p>
{%- endif -%}

{%- include toppage_body.html -%}
