{%- if page.title -%}
    {%- capture page_title -%}{{page.title | xml_escape }} <small>({{ include.data.path | xml_escape }})</small>{%- endcapture -%}
{%- else -%}
    {%- capture page_title -%}{{include.data.path | xml_escape }}{%- endcapture -%}
{%- endif -%}
<h1>{{ site.icons[include.data.verificationStatus] }} {{ page_title | xml_escape }}</h1>

<ul>
    <li><a href="{{ site.github.repository_url }}/blob/{{ site.branch_name | default: "HEAD" }}/{{ site.basedir }}{{ include.data.path }}">View this file on GitHub</a></li>
    {% if code.isFailed %}<li>Link: <a href="{{ site.github.repository_url }}/actions">{{ site.icons[code.verificationStatus] }} View error logs on GitHub Actions</a></li>{%- endif -%}
</ul>
